import pytest
from os import getenv
from dotenv import load_dotenv

from flybook.customBot import CustomBot
from flybook.messageCard import TemplateMessageCard, SimpleMessageCard

load_dotenv(".env.test")


class TestCustomBot:
    """Test cases for the CustomBot class"""

    def setup_method(self):
        public = getenv("CUSTOM_BOT_PUBLIC")
        webhook = getenv("CUSTOM_BOT_WEBHOOK")
        secret = getenv("CUSTOM_BOT_SECRET")
        self.public_bot = CustomBot(public)
        self.secret_bot = CustomBot(webhook, secret)

    def test_send_text(self):
        self.public_bot.send_text("你好，世界！")

    def test_send_simple_card(self):
        card = SimpleMessageCard("你好！", "**世界**", "orange")
        self.public_bot.send_card(card)

    def test_send_template_card(self):
        template_id = getenv("MESSAGE_CARD_TEMPLATE_ID")
        template_version = getenv("MESSAGE_CARD_TEMPLATE_VERSION")
        variable = {
            "title": "你好！",
            "content": "**世界**"
        }
        card = TemplateMessageCard(template_id, template_version, variable)
        self.secret_bot.send_card(card)



import httpx
import hashlib
from base64 import b64encode
from time import time
import hmac

from .common import COMMON_HEADER, checkFeishuResponse
from .messageCard import MessageCard


class CustomBot:
    """
        飞书自定义机器人接口
        参考文档: https://open.feishu.cn/document/client-docs/bot-v3/add-custom-bot

        :param webhook: 自定义机器人的webhook地址
        :param secret?: 自定义机器人的secret, 用于签名验证, 默认为None
    """

    def __init__(self, webhook: str, secret: str | None = None):
        self.webhook = webhook
        self.secret = secret

    def send(self, message: dict):
        """
            发送自定义内容到飞书自定义机器人

            :param message: 要发送的消息, 必须是一个字典，格式参考官方文档
        """
        message = self._prepare(message)
        response = httpx.post(self.webhook, json=message,
                              headers=COMMON_HEADER)
        checkFeishuResponse(response)

    def send_text(self, text: str):
        """
            发送文本消息到飞书自定义机器人

            :param text: 要发送的文本内容
        """
        self.send({
            'msg_type': 'text',
            'content':
            {
                'text': text
            }
        })

    def send_card(self, card: MessageCard):
        """
            发送模板消息到飞书自定义机器人

            :param card: 要发送的模板消息卡片
        """
        self.send({
            'msg_type': 'interactive',
            'card': card.cardData
        })

    def _prepare(self, message: dict) -> str:
        if self.secret:
            timestamp = int(time())
            sign = f"{timestamp}\n{self.secret}"
            sign = hmac.new(
                sign.encode("utf-8"),
                digestmod=hashlib.sha256).digest()
            sign = b64encode(sign).decode("utf-8")
            message['timestamp'] = timestamp
            message['sign'] = sign
        return message



import pytest
from os import getenv
from dotenv import load_dotenv

from flybook.bot import Bot
from flybook.user import User
from flybook.messageCard import SimpleMessageCard

load_dotenv(".env.test")


class TestMessageSender:
    def setup_method(self):
        app_id = getenv("BOT_APP_ID")
        app_secret = getenv("BOT_APP_SECRET")
        self.bot = Bot(app_id, app_secret)

    def test_send_text_message(self):
        tenant_access_token = getenv("BOT_TENANT_ACCESS_TOKEN")
        receiver = User(getenv("USER_ID"))
        self.bot.send_text_message(receiver, "Hello, World!")

    def test_send_card_message(self):
        receiver = User(getenv("GROUP_ID"))
        card = SimpleMessageCard("你好！", "**世界**", "orange")
        self.bot.send_card_message(receiver, card)



from datetime import datetime

from ..common import COMMON_HEADER, checkFeishuResponse
from .prototype import BotPrototype
from .messageSender import MessageSender


class Bot(MessageSender):
    """
        飞书机器人类
    """



import httpx
import json

from ..common import COMMON_HEADER, checkFeishuResponse
from .prototype import BotPrototype
from ..messageCard import MessageCard
from ..user import User


class MessageSender(BotPrototype):
    """
        飞书消息发送类
    """

    url = "https://open.feishu.cn/open-apis/im/v1/messages"

    def send_message(self, receiver: User,
                     msg_type: str, content: dict):
        """
            发送消息到飞书机器人

            :param receiver: 接收者对象
            :param msg_type: 消息类型
            :param content: 消息内容
        """
        message = {
            'receive_id': receiver.id,
            'msg_type': msg_type,
            'content': json.dumps(content)
        }
        headers = {
            **COMMON_HEADER,
            'Authorization': f'Bearer {self.tenant_access_token}'
        }
        response = httpx.post(
            self.url,
            headers=headers,
            json=message,
            params={'receive_id_type': receiver.type}
        )
        checkFeishuResponse(response)

    def send_text_message(self, receiver: User, text: str):
        """
            发送文本消息到飞书机器人

            :param receiver: 接收者对象
            :param text: 文本内容
        """
        self.send_message(receiver, 'text', {'text': text})

    def send_card_message(self, receiver: User, card: MessageCard):
        """
            发送卡片消息到飞书机器人

            :param receiver: 接收者对象
            :param card: 卡片内容
        """
        self.send_message(receiver, 'interactive', card.cardData)



import pytest
from os import getenv
from dotenv import load_dotenv
from datetime import datetime
from uuid import uuid4

from flybook.bot import Bot
from flybook.user import User
from flybook.bitable import Bitable, Table

load_dotenv(".env.test")


class TestBitable:
    def setup_method(self):
        app_id = getenv("BOT_APP_ID")
        app_secret = getenv("BOT_APP_SECRET")
        self.bot = Bot(app_id, app_secret)
        self.bitable = Bitable(app_token=getenv("BITABLE_APP_TOKEN"))
        self.table = Table(self.bitable, getenv("BITABLE_TABLE_ID"))

    def test_add_record(self):
        group = getenv("GROUP_ID")
        user = User(getenv("USER_ID"))
        fields = {
            "文本": "测试记录",
            "数字": 123.456,
            "单选": "good",
            "多选": ["a", "b", "d"],
            "日期": datetime.now().timestamp() * 1000,
            "复选框": True,
            "群组": [{"id": group}],
            "人员": [{"id": user.id}],
            "超链接": {
                "text": "飞书多维表格官网",
                "link": "https://www.feishu.cn/product/base"
            },
        }
        self.table.add_record(self.bot.tenant_access_token,
                              fields,
                              user.type)

    def test_update_record(self):
        record_id = getenv("BITABLE_RECORD_ID")
        fields = {
            "文本": str(uuid4()),
        }
        self.table.update_record(self.bot.tenant_access_token,
                                 record_id,
                                 fields)



class MessageCard:
    """
        飞书消息卡片的基类

        :param cardData: 消息卡片的原始字典数据
    """

    def __init__(self, cardData: dict):
        self._cardData = cardData

    @property
    def cardData(self) -> dict:
        """
            获取消息卡片的原始字典数据

            :return: 消息卡片的原始字典数据
        """
        return self._cardData


class TemplateMessageCard(MessageCard):
    """
        基于模板的飞书卡片

        :param id: 消息卡片的模板ID
        :param version: 消息卡片的模板版本
        :param variables?: 消息卡片的模板参数，默认为空字典
    """

    def __init__(self, id: str, version: str, variable: dict | None = None):
        self.id = id
        self.version = version
        self.variable = variable or {}

    @property
    def cardData(self) -> dict:
        """
            获取消息卡片的原始字典数据

            :return: 消息卡片的原始字典数据
        """
        return {
            "type": "template",
            "data": {
                "template_id": self.id,
                "template_version_name": self.version,
                "template_variable": self.variable
            }
        }


class SimpleMessageCard(MessageCard):
    """
        简单的飞书卡片

        :param title: 卡片的标题
        :param text: 卡片的文本内容
    """

    def __init__(self, title: str, content: str, titleColor: str = "blue"):
        self.title = title
        self.content = content
        self.titleColor = titleColor

    @property
    def cardData(self) -> dict:
        """
            获取消息卡片的原始字典数据

            :return: 消息卡片的原始字典数据
        """
        return {
            "schema": "2.0",
            "header": {
                "title": {
                    "tag": "lark_md",
                    "content": self.title
                },
                "template": self.titleColor
            },
            "body": {
                "elements": [
                    {
                        "tag": "markdown",
                        "content": self.content
                    }
                ]
            }
        }



import httpx

from .bitable import Bitable
from ..common import checkFeishuResponse, COMMON_HEADER


class Table:
    def __init__(self, bitable: Bitable, id: str):
        self.bitable = bitable
        self.id = id

    def add_record(self, access_token: str, fields: dict,
                   user_id_type: str | None = None):
        """
            添加记录

            :param access_token: 飞书机器人访问令牌
            :param fields: 记录字段
            :param user_id_type: 用户ID类型，需与Record中的用户ID类型一致
        """
        url = f"https://open.feishu.cn/open-apis/bitable/v1/apps/{self.bitable.app_token}/tables/{self.id}/records"
        headers = {
            **COMMON_HEADER,
            "Authorization": f"Bearer {access_token}"
        }
        params = {}
        if user_id_type:
            params['user_id_type'] = user_id_type
        response = httpx.post(url, headers=headers, params=params,
                              json={"fields": fields})
        response = checkFeishuResponse(response)
        return response["data"]["record"]["id"]

    def update_record(self, access_token: str, record_id: str, fields: dict,
                      user_id_type: str | None = None):
        """
            更新记录

            :param access_token: 飞书机器人访问令牌
            :param record_id: 记录ID
            :param fields: 记录字段
            :param user_id_type: 用户ID类型，需与Record中的用户ID类型一致
        """
        url = f"https://open.feishu.cn/open-apis/bitable/v1/apps/{self.bitable.app_token}/tables/{self.id}/records/{record_id}"
        headers = {
            **COMMON_HEADER,
            "Authorization": f"Bearer {access_token}"
        }
        params = {}
        if user_id_type:
            params['user_id_type'] = user_id_type
        response = httpx.put(url, headers=headers, params=params,
                             json={"fields": fields})
        checkFeishuResponse(response)



class User:
    """
        用户类

        :param id: 用户ID
    """

    def __init__(self, id: str):
        self.id = id

    @property
    def type(self):
        if self.id.startswith('oc_'):
            return 'chat_id'
        elif self.id.startswith('ou_'):
            return 'union_id'
        elif self.id.startswith('ou_'):
            return 'open_id'
        return 'user_id'



import httpx


COMMON_HEADER = {
    'Content-Type': 'application/json; charset=utf-8'
}


class FeishuAPIException(Exception):
    """
        飞书API异常
    """


def checkFeishuResponse(response: httpx.Response):
    """
        检查飞书API响应是否成功
    """
    try:
        response.raise_for_status()
    except:
        raise FeishuAPIException(response.text)
    response = response.json()
    if response['code'] != 0:
        raise FeishuAPIException(response['msg'])
    return response



import httpx

from ..common import COMMON_HEADER, checkFeishuResponse


class BotPrototype:
    """
        飞书机器人原型类

        :param app_id: 飞书应用ID
        :param app_secret: 飞书应用密钥
    """

    def __init__(self, app_id: str, app_secret: str):
        self.app_id = app_id
        self.app_secret = app_secret
        self.app_access_token = ""
        self.get_tenant_access_token()

    def get_tenant_access_token(self):
        """
            获取飞书租户访问令牌
        """
        response = httpx.post(
            'https://open.feishu.cn/open-apis/auth/v3/tenant_access_token/internal/',
            json={
                'app_id': self.app_id,
                'app_secret': self.app_secret
            }
        )
        response = checkFeishuResponse(response)
        self.tenant_access_token = response['tenant_access_token']
